package id.co.sangkuriang.inovasi.daerah.master.config

import id.co.sangkuriang.inovasi.daerah.master.constant.AUDIENCE_FILTER_PATH
import id.co.sangkuriang.inovasi.daerah.master.interceptor.TokenInterceptor
import org.springframework.boot.web.servlet.FilterRegistrationBean
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.web.servlet.config.annotation.CorsRegistry
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer


@Configuration
class SpringWebConfig : WebMvcConfigurer {

    override fun addCorsMappings(registry: CorsRegistry) {
        registry.addMapping("/**")
            .allowedMethods("HEAD", "GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "PUT", "TRACE")
            .allowedOrigins("*")
            .allowedHeaders("*")
            .maxAge(60000)
    }

    @Bean
    fun tokenFilter(): FilterRegistrationBean<TokenInterceptor>? {
        val registrationBean: FilterRegistrationBean<TokenInterceptor> =
            FilterRegistrationBean<TokenInterceptor>()

        AUDIENCE_FILTER_PATH.forEach {
            it.value.forEach { it2 ->
                registrationBean.addUrlPatterns(it2)
            }
        }

        registrationBean.filter = TokenInterceptor()
        registrationBean.order = 1
        return registrationBean
    }

    override fun addViewControllers(registry: ViewControllerRegistry) {
        registry.addRedirectViewController("/v2/api-docs", "/v2/api-docs")
        registry.addRedirectViewController("/swagger-resources/configuration/ui", "/swagger-resources/configuration/ui")
        registry.addRedirectViewController(
            "/swagger-resources/configuration/security",
            "/swagger-resources/configuration/security"
        )
        registry.addRedirectViewController("/swagger-resources", "/swagger-resources")
    }

    override fun addResourceHandlers(registry: ResourceHandlerRegistry) {
        registry.addResourceHandler("/swagger-ui.html**")
            .addResourceLocations("classpath:/META-INF/resources/swagger-ui.html")
        registry.addResourceHandler("/webjars/**").addResourceLocations("classpath:/META-INF/resources/webjars/")
    }
}
