package id.co.sangkuriang.inovasi.daerah.master.controller

import id.co.sangkuriang.inovasi.daerah.master.model.request.DeleteDataRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.PaginationRequest
import id.co.sangkuriang.inovasi.daerah.master.model.response.ReturnData
import id.co.sangkuriang.inovasi.daerah.master.pojo.Tag
import id.co.sangkuriang.inovasi.daerah.master.service.TagService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.servlet.http.HttpServletRequest
import javax.validation.Valid

@RestController
@RequestMapping(path = ["tag"])
class TagController {

    @Autowired
    private lateinit var service: TagService

    @GetMapping(value = [""], produces = ["application/json"])
    fun getAll(page: PaginationRequest): ResponseEntity<ReturnData>? {
        return service.getAll(page)
    }

    @GetMapping(value = ["{id}"], produces = ["application/json"])
    fun getDetail(@PathVariable id: String): ResponseEntity<ReturnData>? {
        return service.getDetail(id)
    }

    @PostMapping(value = [""], produces = ["application/json"])
    fun saveData(@Valid @RequestBody request: Tag): ResponseEntity<ReturnData>? {
        return service.saveData(request)
    }

    @PutMapping(value = ["{id}"], produces = ["application/json"])
    fun updateData(@PathVariable id: String, @Valid @RequestBody request: Tag): ResponseEntity<ReturnData>? {
        return service.updateData(id, request)
    }

    @DeleteMapping(value = [""], produces = ["application/json"])
    fun deleteData(
        @Valid @RequestBody request: DeleteDataRequest,
        httpServletRequest: HttpServletRequest
    ): ResponseEntity<ReturnData>? {
        return service.deleteData(request, httpServletRequest)
    }
}
