package id.co.sangkuriang.inovasi.daerah.master.service.impl

import id.co.sangkuriang.inovasi.daerah.master.constant.HEADER_STRING
import id.co.sangkuriang.inovasi.daerah.master.constant.VALIDATOR_MSG_DELETE_RELATION
import id.co.sangkuriang.inovasi.daerah.master.constant.VALIDATOR_MSG_HAS_USED
import id.co.sangkuriang.inovasi.daerah.master.constant.VALIDATOR_MSG_REQUEST_FAILED
import id.co.sangkuriang.inovasi.daerah.master.exception.BadRequestException
import id.co.sangkuriang.inovasi.daerah.master.helpers.*
import id.co.sangkuriang.inovasi.daerah.master.model.request.DeleteDataRequest
import id.co.sangkuriang.inovasi.daerah.master.model.request.PaginationRequest
import id.co.sangkuriang.inovasi.daerah.master.model.response.ReturnData
import id.co.sangkuriang.inovasi.daerah.master.pojo.Regency
import id.co.sangkuriang.inovasi.daerah.master.repository.RegencyRepository
import id.co.sangkuriang.inovasi.daerah.master.service.RegencyService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.domain.Page
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Pageable
import org.springframework.data.domain.Sort
import org.springframework.http.ResponseEntity
import org.springframework.stereotype.Service
import java.util.*
import javax.servlet.http.HttpServletRequest

@Suppress("NAME_SHADOWING")
@Service
class RegencyServiceImpl : RegencyService {

    private val title = "Kabupaten/Kota"

    @Autowired
    private lateinit var repo: RegencyRepository

    override fun getAll(page: PaginationRequest): ResponseEntity<ReturnData> {
        return try {
            val page: Page<Regency?>? = if (page.enablePage!! == 1) {
                var direction: Sort.Direction = Sort.Direction.ASC
                if (page.sort?.lowercase(Locale.getDefault()).equals("desc")) {
                    direction = Sort.Direction.DESC
                }
                repo.getListPagination(
                    param = page.param.toString().lowercase(Locale.getDefault()),
                    pageable = PageRequest.of(page.page!!, page.size!!, Sort.by(direction, page.sortBy))
                )
            } else {
                repo.getListPagination(
                    param = page.param.toString().lowercase(Locale.getDefault()),
                    pageable = Pageable.unpaged()
                )
            }
            responseSuccess(data = page)
        } catch (e: Exception) {
            throw e
        }
    }

    override fun getDetail(id: String): ResponseEntity<ReturnData> {
        try {
            val data = repo.findByIdAndActive(id)
            if (data.isPresent) {
                return responseSuccess(data = data)
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    override fun saveData(request: Regency): ResponseEntity<ReturnData> {
        return try {
            validateRequest(request)
            val save = Regency()
            save.name = request.name
            save.provinceId = request.provinceId

            responseCreated(data = repo.save(save))
        } catch (e: Exception) {
            throw e
        }
    }

    override fun updateData(id: String, request: Regency): ResponseEntity<ReturnData> {
        try {
            val data = repo.findByIdAndActive(id)
            if (data.isPresent) {
                validateRequest(request, id)
                val update = data.get()
                update.name = request.name
                update.updatedAt = Date()

                return responseSuccess(data = repo.save(update))
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    override fun deleteData(request: DeleteDataRequest, httpServletRequest: HttpServletRequest): ResponseEntity<ReturnData> {
        try {
            val data = repo.findByIdAndActive(request.id)
            if (data.isPresent) {
                val checkApplication: ReturnData =
                    RequestHelpers.applicationCountByParam(
                        httpServletRequest.getHeader(HEADER_STRING),
                        "regencyId",
                        data.get().id
                    ) as ReturnData
                if (checkApplication.success == true) {
                    if (checkApplication.data != null) {
                        val countApplication: Long = (checkApplication.data as Int).toLong()
                        if (countApplication > 0) {
                            return responseBadRequest("$VALIDATOR_MSG_DELETE_RELATION $title ${data.get().name} mempunyai $countApplication data Aplikasi")
                        }
                    }
                } else {
                    return responseBadRequest("$VALIDATOR_MSG_REQUEST_FAILED Aplikasi")
                }

                val update = data.get()
                update.active = false
                update.updatedAt = Date()
                repo.save(update)
                return responseSuccess()
            }
            return responseNotFound()
        } catch (e: Exception) {
            throw e
        }
    }

    private fun validateRequest(request: Regency, id: String? = null) {
        try {
            val listMessage = mutableListOf<Map<String, String>>()
            val checkName = repo.findByNameProvinceId(request.name, request.provinceId)

            if (checkName.isPresent) {
                val msgCheckName = mapOf("name" to "Nama ${request.name} $VALIDATOR_MSG_HAS_USED")
                if (id == null) {
                    listMessage.add(msgCheckName)
                } else {
                    if (checkName.get().id != id) {
                        listMessage.add(msgCheckName)
                    }
                }
            }

            if (listMessage.isNotEmpty()) {
                throw BadRequestException(data = listMessage)
            }
        } catch (e: Exception) {
            throw e
        }
    }
}
